import es
import playerlib
from helpers import *

db_cursor = None

# == ACHIEVEMENT CLASS ==================================
class Achievement:
    __parentID = None
    __id = None
    __name = None
    __description = None
    __totalProgress = None
    __playerProgress = {}
    
    def __init__(self, newParentID, newID, newName, newDescription, newTotalProgress = 1):
        self.__parentID = newParentID
        self.__id = newID
        self.__name = newName
        self.__description = newDescription
        self.__totalProgress = int(newTotalProgress)
        self.__playerProgress = {}
        if db_cursor:
            db_cursor.execute('INSERT INTO achievements (id, groupid, name, description, totalprogress) VALUES ("%s", "%s", "%s", "%s", "%d")' % (newID, newParentID, newName, newDescription, int(newTotalProgress)))        

    def __getattr__(self, name):
        if name == "parentid":
            return self.__parentID
        elif name == "id":
            return self.__id
        elif name == "name":
            return self.__name
        elif name == "description":
            return self.__description
        elif name == "totalprogress":
            return int(self.__totalProgress)
        elif name in self.__dict__.keys():
            return self.__dict__[name]
        else:
            raise AttributeError, name
    
    def __setattr__(self, name, value):
        if name == "id":
            if db_cursor:
                db_cursor.execute('UPDATE achievements SET id="%s" WHERE id="%s" AND groupid="%s"' % (value, self.__id, self.__parentID))
            self.__id = value
        elif name == "name":
            if db_cursor:
                db_cursor.execute('UPDATE achievements SET name="%s" WHERE id="%s" AND groupid="%s"' % (value, self.__id, self.__parentID))
            self.__name = value
        elif name == "description":
            if db_cursor:
                db_cursor.execute('UPDATE achievements SET description="%s" WHERE id="%s" AND groupid="%s"' % (value, self.__id, self.__parentID))
            self.__description = value
        elif name == "totalprogress":
            if db_cursor:
                db_cursor.execute('UPDATE achievements SET totalprogress="%d" WHERE id="%s" AND groupid="%s"' % (int(value), self.__id, self.__parentID))
            self.__totalProgress = int(value)
        else:
            self.__dict__[name] = value

    def setProgressForPlayer(self, playerID, progress):
        steamID = getUniqueSteam(playerID)
        self.__playerProgress[steamID] = progress
        self.__databaseSetProgress(steamID, progress)

    def stepProgressForPlayer(self, playerID, stepAmount = 1):
        backwardStep = False
        steamID = getUniqueSteam(playerID)
        
        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])
            
        if not progress == None: 
            if progress == self.__totalProgress:
                if stepAmount >= 0: 
                    return None
                else:
                    backwardStep = True
            progress = progress + stepAmount
        else:
            progress = stepAmount
        
        if progress < 0:
            progress = 0
        if progress >= self.__totalProgress:
            progress = self.__totalProgress
            self.__doAchievementPlayer(playerID)
        elif backwardStep:
            self.__undoAchievementPlayer(playerID)
            
        self.__playerProgress[steamID] = progress
        self.__databaseSetProgress(steamID, progress)
    
    def stepProgressForList(self, listFilter, stepAmount = 1):
        for ply in playerlib.getUseridList(listFilter):
            self.stepProgressForPlayer(ply, stepAmount)
    
    def stepProgressForAllActive(self, stepAmount = 1):
        for ply in es.getUseridList():
            self.stepProgressForPlayer(ply, stepAmount)
            
    def completeProgressForPlayer(self, playerID):
        steamID = getUniqueSteam(playerID)

        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])

        if (not progress) or progress < self.__totalProgress:
            self.__playerProgress[steamID] = self.__totalProgress
            self.__databaseSetProgress(steamID, self.__totalProgress)
            self.__doAchievementPlayer(playerID)
    
    def completeProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.completeProgressForPlayer(ply)
                
    def completeProgressForAllActive(self):
        for ply in es.getUseridList():
            self.completeProgressForPlayer(ply)
                
    def resetProgressForPlayer(self, playerID):
        steamID = getUniqueSteam(playerID)

        progress = None
        if steamID in self.__playerProgress.keys():
            progress = int(self.__playerProgress[steamID])
            
        self.__playerProgress[steamID] = 0
        self.__databaseSetProgress(steamID, 0)
        if progress and progress == self.__totalProgress:
            self.__resetAchievementPlayer(playerID)
       
    def resetProgressForList(self, listFilter):
        for ply in playerlib.getUseridList(listFilter):
            self.resetProgressForPlayer(ply, stepAmount)
                
    def resetProgressForAllActive(self):
        for ply in es.getUseridList():
            self.resetProgressForPlayer(ply, stepAmount)
                                          
    def getPlayerProgress(self, playerID):
        steamID = getUniqueSteam(playerID)
        if "STEAM" in playerID:
            steamID = playerID
            
        if steamID in self.__playerProgress.keys():
            return int(self.__playerProgress[steamID])
        return 0
    
    def isPlayerCompleted(self, playerID):
        return self.getPlayerProgress(playerID) == self.__totalProgress
            
    def __databaseSetProgress(self, steamID, progress):
        if db_cursor:
            db_cursor.execute('SELECT * FROM achievements_progress WHERE steamid="%s" AND achievementid="%s" AND groupid="%s"' % (steamID, self.__id, self.__parentID))
            exists = False
            for row in db_cursor.fetchall():
                exists = True
                break
            
            if exists:
                db_cursor.execute('UPDATE achievements_progress SET progress="%d" WHERE steamid="%s" AND achievementid="%s" AND groupid="%s"' % (int(progress), steamID, self.__id, self.__parentID))
            else:
                db_cursor.execute('INSERT INTO achievements_progress (steamid, achievementid, groupid, progress) VALUES ("%s", "%s", "%s", "%d")' % (steamID, self.__id, self.__parentID, int(progress)))

    def __doAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_achieved", playerID)     
    
    def __undoAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_unachieved", playerID)

    def __resetAchievementPlayer(self, playerID):
        self.__triggerEvent("achievement_reset", playerID)
    
    def __triggerEvent(self, eventName, playerID):
        es.event("initialize", eventName)
        es.event("setstring", eventName, "achievementgroupid", self.__parentID)
        es.event("setstring", eventName, "achievementid", self.__id)
        es.event("setstring", eventName, "playerid", playerID)
        es.event("fire", eventName)
# =======================================================
